X-Git-Url: http://git.phpeclipse.com

diff --git a/net.sourceforge.phpeclipse.phpmanual/src/net/sourceforge/phpeclipse/phpmanual/views/PHPManualView.java b/net.sourceforge.phpeclipse.phpmanual/src/net/sourceforge/phpeclipse/phpmanual/views/PHPManualView.java
index 09233d3..e14af65 100644
--- a/net.sourceforge.phpeclipse.phpmanual/src/net/sourceforge/phpeclipse/phpmanual/views/PHPManualView.java
+++ b/net.sourceforge.phpeclipse.phpmanual/src/net/sourceforge/phpeclipse/phpmanual/views/PHPManualView.java
@@ -7,6 +7,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
 import java.util.ArrayList;
+import java.util.regex.Matcher;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipFile;
 
@@ -29,6 +30,7 @@ import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.browser.Browser;
 import org.eclipse.ui.IEditorPart;
+import org.eclipse.ui.INullSelectionListener;
 import org.eclipse.ui.ISelectionListener;
 import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.part.ViewPart;
@@ -57,7 +59,7 @@ import org.osgi.framework.Bundle;
  * <p>
  * @author scorphus
  */
-public class PHPManualView extends ViewPart implements ISelectionListener, ISelectionListenerWithAST {
+public class PHPManualView extends ViewPart implements INullSelectionListener, ISelectionListenerWithAST {
 
 	/**
 	 * The ViewPart's browser
@@ -71,6 +73,11 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 	private PHPEditor lastEditor;
 
 	/**
+	 * String that stores the last selected word
+	 */
+	private String lastOccurrence = null;
+
+	/**
 	 * The path to the doc.zip file containing the PHP Manual
 	 * in HTML format
 	 */
@@ -95,7 +102,7 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 			SelectionListenerWithASTManager.getDefault().addListener(lastEditor, this);
 		}
 		getSite().getWorkbenchWindow().getSelectionService()
-				.addSelectionListener(PHPeclipsePlugin.EDITOR_ID, this);
+				.addPostSelectionListener(PHPeclipsePlugin.EDITOR_ID, this);
 	}
 
 	/**
@@ -103,7 +110,7 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 	 */
 	public void dispose() {
 		getSite().getWorkbenchWindow().getSelectionService()
-				.removeSelectionListener(PHPeclipsePlugin.EDITOR_ID, this);
+				.removePostSelectionListener(PHPeclipsePlugin.EDITOR_ID, this);
 	}
 
 	/**
@@ -120,12 +127,17 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 		IDocument document = ((PHPEditor)part).getViewer().getDocument();
 		int offset = selection.getOffset();
 		IRegion iRegion = JavaWordFinder.findWord(document, offset);
-		try {
-			final String wordStr = document.get(iRegion.getOffset(),
-					iRegion.getLength());
-			showReference(wordStr);
-		} catch (Exception e) {
-			e.printStackTrace();
+		if (document != null && iRegion != null) {
+			try {
+				final String wordStr = document.get(iRegion.getOffset(),
+						iRegion.getLength());
+				if (!wordStr.equalsIgnoreCase(lastOccurrence)) {
+					showReference(wordStr);				
+					lastOccurrence = wordStr;
+				}
+			} catch (Exception e) {
+				e.printStackTrace();
+			}
 		}
 	}
 
@@ -134,9 +146,11 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 	 * instance of PHPEditor it gets a listener attached
 	 */
 	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
-		if (!((PHPEditor)part).equals(lastEditor)) {
+		if (part != null && !((PHPEditor)part).equals(lastEditor)) {
 			SelectionListenerWithASTManager.getDefault().addListener((PHPEditor)part, this);
 			lastEditor = (PHPEditor)part;
+		} else {
+			System.out.println(part);
 		}
 	}
 
@@ -146,7 +160,6 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 	 * @param funcName Function name
 	 */
 	private void showReference(final String funcName) {
-		System.out.println("Show reference for " + funcName);
 		new Thread(new Runnable() {
 			public void run() {
 				Display.getDefault().asyncExec(new Runnable() {
@@ -169,7 +182,7 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 		try {
 			Parser parser = new Parser(source);
 			String [] tagsToBeFound = {"DIV"};
-			ArrayList<String> classList = new ArrayList<String>(6);
+			ArrayList classList = new ArrayList(8);
 			classList.add("refnamediv");
 			classList.add("refsect1 description");
 			classList.add("refsect1 parameters");
@@ -180,14 +193,14 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 			TagFindingVisitor visitor = new TagFindingVisitor(tagsToBeFound);
 			parser.visitAllNodesWith(visitor);
 			Node [] allPTags = visitor.getTags(0);
-			StringBuilder output = new StringBuilder();
+			StringBuffer output = new StringBuffer();
 			for (int i = 0; i < allPTags.length; i++) {
 				String tagClass = ((Div)allPTags[i]).getAttribute("class");
 				if (classList.contains(tagClass)) {
 					output.append(allPTags[i].toHtml());
 				}
 			}
-			return output.toString().replace("—", "-");
+			return output.toString().replaceAll("—", "-");
 			//.replace("<h3 class=\"title\">Description</h3>", " ");
 		} catch (ParserException e) {
 			e.printStackTrace();
@@ -233,10 +246,38 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 	}
 
 	/**
-	 * Looks for the function's reference page inside the doc.zip file
-	 * and returns a filtered HTML source of it embedded in the template
-	 *  
-	 * @param funcName Function name
+	 * Replaces each substring of source string that matches the
+	 * given pattern string with the given replace string
+	 * 
+	 * @param source The source string
+	 * @param pattern The pattern string
+	 * @param replace The replace string
+	 * @return The resulting String
+	 */
+	public static String replace(String source, String pattern, String replace) {
+		if (source != null) {
+			final int len = pattern.length();
+			StringBuffer sb = new StringBuffer();
+			int found = -1;
+			int start = 0;
+			while ((found = source.indexOf(pattern, start)) != -1) {
+				sb.append(source.substring(start, found));
+				sb.append(replace);
+				start = found + len;
+			}
+			sb.append(source.substring(start));
+			return sb.toString();
+		} else {
+			return "";
+		}
+	}
+
+	/**
+	 * Looks for the function's reference page inside the doc.zip file and
+	 * returns a filtered HTML source of it embedded in the template
+	 * 
+	 * @param funcName
+	 *            Function name
 	 * @return HTML source of reference page
 	 */
 	public String getHtmlSource(String funcName) {
@@ -250,15 +291,17 @@ public class PHPManualView extends ViewPart implements ISelectionListener, ISele
 			InputStream ref = docFile.getInputStream(entry);
 			b = new byte[(int)entry.getSize()];
 			ref.read(b, 0, (int)entry.getSize());
+			if (b != null) {
+				String reference = filterHtmlSource(new String(b));
+				String refPageTpl = getRefPageTemplate();
+				refPageTpl = refPageTpl.replaceAll("%title%", funcName);
+				refPageTpl = replace(refPageTpl, "%reference%", reference);
+				return refPageTpl;
+			}
 		} catch (IOException e) {
 			return "<html></html>";
-		}
-		if (b != null) {
-			String reference = filterHtmlSource(new String(b));
-			String refPageTpl = getRefPageTemplate();
-			refPageTpl = refPageTpl.replace("{title}", funcName);
-			refPageTpl = refPageTpl.replace("{reference}", reference);
-			return refPageTpl;
+		} catch (Exception e) {
+			return null;
 		}
 		return "<html></html>";
 	}